#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include <EEPROM.h>

#define ONE_WIRE_BUS    D7
#define LUEFTER_PWM     D1
#define TEMPERATURE_PRECISION 12
#define MIN_PWM 300
#define MAX_PWM 900

const char htmlhead[] PROGMEM = R"=====(
<!DOCTYPE HTML>
<html><head>
<meta http-equiv=Content-Type content="text/html; charset=utf-8" />
<meta name=viewport content="width=device-width, initial-scale=1.0">
<style>
* {
font-family: "Lucida Sans Unicode", "Lucida Grande", sans-serif;
}

html, body {
  margin: 0;
  padding: 0;
  background: #2f2f2f;
}

h1 {
  display: block;
  padding: 1em 0.5em;
  color: #FFFFFF;
  margin: 0;
  font-size: 2em;
  background: #017db9;
  font-weight: normal;
  text-align: center;
}

input[type="submit"] {
  border: 1px solid #017db9;
  background: #017db9;
  border-radius: 4px;
  -webkit-box-shadow: 0px 4px 0px 0px #006fa4;
  -moz-box-shadow: 0px 4px 0px 0px #006fa4;
  box-shadow: 0px 4px 0px 0px #006fa4;
  color: #ffffff;
  font-size: 1em;
  padding: 0.5em;
  margin: 0.5em;
}

div.wrap {
  background: #fff;
  border: 2px solid #007ebb;
  margin: 1em;
  padding: 0.5em;
  text-align: center;
}

select {
  border: 1px solid #CCCCCC;
  padding: 0.5em;
  font-size: 1em;
  text-align: center;
  width: 6em;
}

label, select {
  display: inline-block;
  margin-bottom: 0.5em;
}

label {
  text-align: right;
  padding: 0.5em 1em 0.5em 0.5em;
  width: 6em;
}

h2 {
  margin: 0 0.5em 0.5em 0.5em;
  color: #333333;
  font-weight: normal;
  font-size:1.5em;
}

.clear {
  clear: left;
}
</style>
</head>
<body>
<h1>L&uuml;fter Steuerung</h1>
)=====";

/* Set these to your desired credentials. */
const char *ssid = "LuefterSteuerung";
const char *password = "12345678";

OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);
DeviceAddress Sensor0;

IPAddress local_IP(192,168,1,1);
IPAddress gateway(192,168,1,2);
IPAddress subnet(255,255,255,0);

ESP8266WebServer server(80);
uint8_t TempThresholdMin = 0;
uint8_t TempThresholdMax = 0;
float dTemperature = 0;
  uint16_t u16PwmSetpoint = 0;

String strOption(uint16_t selected, uint16_t start, uint16_t ende);
void vControlFans(uint8_t u8Pwm);

void handleRoot() {
  uint8_t NewTempThresholdMin = 0;
  uint8_t NewTempThresholdMax = 0;

  String antwort = "";
  if (server.args() > 0 )
  {
    for (uint8_t i = 0; i < server.args(); i++ )
    {
      if(server.argName(i).indexOf("mintemp") != -1)
        NewTempThresholdMin = server.arg(i).toInt();
      else if(server.argName(i).indexOf("maxtemp") != -1)
        NewTempThresholdMax = server.arg(i).toInt();

      antwort+= server.argName(i)+": "+server.arg(i)+"<br>";
    }

// Check if value has changed, update eeprom
    if(NewTempThresholdMin != TempThresholdMin)
    {
      TempThresholdMin = NewTempThresholdMin;
      EEPROM.write(0, TempThresholdMin);
    }

    if(NewTempThresholdMax != TempThresholdMax)
    {
      TempThresholdMax = NewTempThresholdMax;
      EEPROM.write(1, TempThresholdMax);
    }

     EEPROM.commit();
  }
  
  String strSeiteninhalt = "";

  if(antwort != "")
  {
    strSeiteninhalt += "<div class='wrap'><h2>Erfolgreich aktualisiert!</h2><p>";
    strSeiteninhalt += "Einschalttemperatur: ";
    strSeiteninhalt += TempThresholdMin;
    strSeiteninhalt += "°C";
    strSeiteninhalt += "</p></div></body></html>";
  } 
  else 
  {
    strSeiteninhalt += "<form action=/ method=GET><div class='wrap'><label>Einschalttemperatur: </label><br>";
    strSeiteninhalt += "<select name='mintemp' size='1'>";
    strSeiteninhalt += strOption(TempThresholdMin, 15, 30);
    strSeiteninhalt += "</select><br class='clear'>";
    strSeiteninhalt += "<div class='wrap'><input type=submit value=Aktualisieren></div></form><br>";
    strSeiteninhalt += "<p>Aktuelle Temperatur: ";
    strSeiteninhalt += String(dTemperature, 2);
    strSeiteninhalt += "</body></html>";
  }

  server.send(200, "text/html", htmlhead+strSeiteninhalt);
}

void setup() {
  delay(10); 
  EEPROM.begin(2); // Just 2 values needed for temperature thresholds
  TempThresholdMin = EEPROM.read(0); // read threshold out of eeprom
  TempThresholdMax = EEPROM.read(1); // read threshold out of eeprom

  WiFi.softAPConfig(local_IP, gateway, subnet);
  WiFi.softAP(ssid, password);

  server.on("/", handleRoot);
  server.begin();

  sensors.begin();
  sensors.getAddress(Sensor0, 0);
  sensors.setResolution(Sensor0, TEMPERATURE_PRECISION);  
  delay(2000);
  //analogWriteFreq(10000);
 pinMode(LUEFTER_PWM, OUTPUT);
}

void loop() {  
static boolean fTimer = true;

if(millis() > (1000 * 60 * 5))
  {
    fTimer = false;
  }
  
// Deactivate Hotspot after some time
  if(!fTimer)
  {
    WiFi.softAPdisconnect(true);
  }
  else
  {
  // If hotspot is still active, handle server
    server.handleClient();  
  }

  sensors.requestTemperatures();
  delay(600);
  dTemperature = sensors.getTempC(Sensor0);  

  if(dTemperature > TempThresholdMin)
  {
    digitalWrite(LUEFTER_PWM, HIGH);
    //u16PwmSetpoint = map((uint8_t)dTemperature, TempThresholdMin, TempThresholdMax, MIN_PWM, MAX_PWM);
  }
  else if(dTemperature < TempThresholdMin - 2.0) // 2K hysteresis
  {
    digitalWrite(LUEFTER_PWM, LOW);
    //u16PwmSetpoint = 0;
  }

  //vControlFans(u16PwmSetpoint);
}

void vControlFans(uint8_t u8Pwm)
{
  static uint8_t u8OldPwm = 0;
  static uint32_t u32TimeLock = 0;

    // Check if switch on is needed
    if((u8OldPwm == 0)&&(u8Pwm != 0))
    {
      u32TimeLock = millis();
      analogWrite(LUEFTER_PWM ,1023); // full power!
    }

    if(millis() > (u32TimeLock + 1000))
    {
      analogWrite(LUEFTER_PWM, u8Pwm); // Power as wished!  
    }
    u8OldPwm = u8Pwm;
}

String strOption(uint16_t selected, uint16_t start, uint16_t ende)
{
  String retString = "";

  for(int i = start; i <= ende; i++)
  {
    if(i == selected)
    {
      retString += "<option selected>";
      retString += i;
      retString += "</option>";
    }
    else
    {
      retString += "<option>";
      retString += i;
      retString += "</option>";
    }
  }

  return retString;
}
